﻿using System;
using System.Collections.Generic;
using System.Linq;
using Interfaces;

namespace HR
{
    public class HrManager : IHr
    {
        private const string EMP_DOES_NOT_EXIST = "员工不存在";
        private const string EMP_ALREADY_EXISTS = "员工已存在";
        private const string ONLY_PROBATION = "只有试用期员工可转正";
        private const string ONLY_NORMAL = "只有正式员工可以加薪";

        private static EmployeeStatus ParseEmployeeStatus(string status)
        {
            EmployeeStatus enuStatus;
            if (Enum.TryParse(status, true, out enuStatus))
                return enuStatus;
            return EmployeeStatus.Probation;
        }

        public IEnumerable<Employee_Get> GetAllEmployees()
        {
            LinqHrDataContext dc = new LinqHrDataContext();
            return dc.employees.Select(e => new Employee_Get
                                            {
                                                EmpNo = e.emp_no,
                                                Name = e.name,
                                                Birthday = e.birthday,
                                                Descriptions = e.descriptions,
                                                Status = ParseEmployeeStatus(e.status),
                                                Salary = e.salary
                                            });
        }

        public Employee_Get GetEmployee(string empno)
        {
            LinqHrDataContext dc = new LinqHrDataContext();
            employee emp = dc.employees.SingleOrDefault(e => e.emp_no == empno);

            if (emp==null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            return new Employee_Get
                       {
                           EmpNo = emp.emp_no,
                           Name = emp.name,
                           Birthday = emp.birthday,
                           Descriptions = emp.descriptions,
                           Status = ParseEmployeeStatus(emp.status),
                           Salary = emp.salary
                       };
        }

        public void AddEmployee(Employee_Add emp)
        {
            LinqHrDataContext dc = new LinqHrDataContext();
            if (dc.employees.Any(e => e.emp_no == emp.EmpNo))
            {
                throw new DemoException(EMP_ALREADY_EXISTS);
            }
            employee empToAdd = new employee
                                {
                                    emp_no = emp.EmpNo,
                                    name = emp.Name,
                                    birthday = emp.Birthday,
                                    descriptions = emp.Descriptions,
                                    status = EmployeeStatus.Probation.ToString(),
                                    salary = 2000.00m
                                };
            dc.employees.InsertOnSubmit(empToAdd);
            dc.SubmitChanges();
        }

        public void EditEmployee(string empno, Employee_Edit emp)
        {
            LinqHrDataContext dc = new LinqHrDataContext();
            employee empToEdit = dc.employees.SingleOrDefault(e => e.emp_no == empno);

            if (empToEdit==null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            empToEdit.name = emp.Name;
            empToEdit.birthday = emp.Birthday;
            empToEdit.descriptions = emp.Descriptions;
            dc.SubmitChanges();
            
        }

        public void DeleteEmployee(string empno)
        {
            LinqHrDataContext dc = new LinqHrDataContext();
            employee empToDelete = dc.employees.SingleOrDefault(e => e.emp_no == empno);
            if (empToDelete == null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            dc.employees.DeleteOnSubmit(empToDelete);
            dc.SubmitChanges();
        }

        public void BecomeRegular(string empno)
        {
            LinqHrDataContext dc = new LinqHrDataContext();
            employee empToUpdate = dc.employees.SingleOrDefault(e => e.emp_no == empno);
            if (empToUpdate == null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            if (ParseEmployeeStatus(empToUpdate.status) != EmployeeStatus.Probation)
            {
                throw new DemoException(ONLY_PROBATION);
            }
            empToUpdate.status = EmployeeStatus.Normal.ToString();
            dc.SubmitChanges();
        }

        public void BecomeLeave(string empno)
        {
            LinqHrDataContext dc = new LinqHrDataContext();
            employee empToUpdate = dc.employees.SingleOrDefault(e => e.emp_no == empno);

            if (empToUpdate == null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            empToUpdate.status = EmployeeStatus.Leaved.ToString();
            dc.SubmitChanges();
        }

        public void IncreaseSalary(string empno, decimal scope)
        {
            LinqHrDataContext dc = new LinqHrDataContext();
            employee empToUpdate = dc.employees.SingleOrDefault(e => e.emp_no == empno);
            if (empToUpdate == null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            if (ParseEmployeeStatus(empToUpdate.status) != EmployeeStatus.Normal)
            {
                throw new DemoException(ONLY_NORMAL);
            }
            empToUpdate.salary += scope;
            dc.SubmitChanges();
        }

        public bool IsEmployeeExist(string empno)
        {
            LinqHrDataContext dc = new LinqHrDataContext();
            return dc.employees.Any(e => e.emp_no == empno);
        }
    }
}
